module net.sourceforge.argparse4j {
    requires java.base;

    exports net.sourceforge.argparse4j;
    exports net.sourceforge.argparse4j.annotation;
    exports net.sourceforge.argparse4j.helper;
    exports net.sourceforge.argparse4j.impl;
    exports net.sourceforge.argparse4j.impl.action;
    exports net.sourceforge.argparse4j.impl.choice;
    exports net.sourceforge.argparse4j.impl.type;
    exports net.sourceforge.argparse4j.inf;
    exports net.sourceforge.argparse4j.internal;
}
